﻿#include "define.h"

// INCLUDE
	vec4 computePixelColor();
// IN
	uniform int bucketFragments;

#if multisample
	layout(binding = 10		 )			 uniform sampler2DMS		tex_depth_bounds;
	layout(binding = 0, r32ui) coherent  uniform uimage2DMSArray    image_next;
#else
	layout(binding = 10		 )			 uniform sampler2DRect		tex_depth_bounds;
	layout(binding = 0, r32ui) coherent  uniform uimage2DArray		image_next;
#endif 
	layout(binding = 1, rg32f) writeonly uniform  imageBuffer		image_peel;
	layout(binding = 2, r32ui) writeonly uniform uimageBuffer		image_pointers;
	
	layout(binding = 3, offset = 0)		 uniform atomic_uint		next_address_0;
	layout(binding = 4, offset = 0)		 uniform atomic_uint		next_address_1;
	layout(binding = 5, offset = 0)		 uniform atomic_uint		next_address_2;
	layout(binding = 6, offset = 0)		 uniform atomic_uint		next_address_3;
	layout(binding = 7, offset = 0)		 uniform atomic_uint		next_address_4;
//
#if multisample
	uint exchangePixelCurrentPageID	(const ivec3 coords, const uint val)	{ return imageAtomicExchange(image_next, coords, gl_SampleID, val);}
#else
	uint exchangePixelCurrentPageID	(const ivec3 coords, const uint val)	{ return imageAtomicExchange(image_next, coords, val);}
#endif 
	void sharedPoolSetValue			(const uint index  , const vec4 val)	{		 imageStore(image_peel    , int(index), val);}
	void sharedPoolSetLink			(const uint index  , const uint val)	{		 imageStore(image_pointers, int(index), uvec4(val, 0U, 0U, 0U) );}

void main(void)
{
	ivec2 coords = ivec2(gl_FragCoord.xy);

	// Compute Fragment Value
	vec4  value  = vec4(packUnorm4x8(computePixelColor().abgr), gl_FragCoord.z, 0.0f, 0.0f);

	// Find Bucket 
	vec2	depth_bounds;
#if multisample
	depth_bounds = texelFetch(tex_depth_bounds, coords, gl_SampleID).rg;
#else
	depth_bounds = texture	 (tex_depth_bounds, gl_FragCoord.xy).rg;
#endif
	float	depth_near   = -depth_bounds.x;
	float	depth_length =  depth_bounds.y - depth_near;
	int		bucket		 = int(floor(float(BUCKET_SIZE)*((gl_FragCoord.z-depth_near)/depth_length))); if (bucket == BUCKET_SIZE) bucket = BUCKET_SIZE-1;

	// Linked Lists

	// Page ID
	uint page_id, next_id, glob_pID;

	if	   (bucket == 0) page_id = atomicCounterIncrement(next_address_0);
	else if(bucket == 1) page_id = atomicCounterIncrement(next_address_1);
	else if(bucket == 2) page_id = atomicCounterIncrement(next_address_2);
	else if(bucket == 3) page_id = atomicCounterIncrement(next_address_3);
	else				 page_id = atomicCounterIncrement(next_address_4);

	next_id  = exchangePixelCurrentPageID(ivec3(coords,bucket), page_id);
	
	glob_pID = uint(bucketFragments*bucket);
	page_id += glob_pID;
	next_id	+= glob_pID;

	sharedPoolSetLink  (page_id, next_id);
	sharedPoolSetValue (page_id, value  );
}