﻿#include "define.h"

// INCLUDE
	vec4 computePixelColor();
// IN

#if multisample
#if multipass
	layout(binding = 3)					 uniform  sampler2DMS	 tex_depth;
#endif
	layout(binding = 0, r32ui) coherent  uniform uimage2DMS	     image_counter;
	layout(binding = 1, rg32f) writeonly uniform  image2DMSArray image_peel;	
#else
#if multipass
	layout(binding = 3)					 uniform  sampler2DRect	 tex_depth;
#endif
	layout(binding = 0, r32ui) coherent  uniform uimage2DRect	 image_counter;
	layout(binding = 1, rg32f) writeonly uniform  image2DArray	 image_peel;
#endif

#if multisample
#if multipass	
	float getPixelDepth			(const ivec2 coords)				{texelFetch(tex_depth, coords, gl_SampleID).r;}
#endif
	uint  addPixelFragCounter	(const ivec2 coords				   ){return imageAtomicIncWrap	(image_counter, coords, gl_SampleID, FAB_SIZE);}
	void  setPixelFragValue		(const ivec3 coords, const vec4 val){		imageStore			(image_peel   , coords, gl_SampleID, val);}
#else
#if multipass
	float getPixelDepth			(const ivec2 coords)				{texture	(tex_depth, gl_FragCoord.xy).r;}
#endif
	uint  addPixelFragCounter	(const ivec2 coords				   ){return imageAtomicIncWrap	(image_counter, coords, FAB_SIZE);}
	void  setPixelFragValue		(const ivec3 coords, const vec4 val){		imageStore			(image_peel   , coords, val);}
#endif

void main(void)
{
	ivec2 coords = ivec2(gl_FragCoord.xy);

#if multipass
	float depth = getPixelDepth(coords);
	if(gl_FragCoord.z <= depth)
		discard;
#endif

	int   id;
#if trimless || trimming || collision
	float frontZ = (gl_FrontFacing) ? gl_FragCoord.z : -gl_FragCoord.z;
	vec4  value = vec4(packUnorm4x8(computePixelColor().abgr), frontZ		 , 0.0f, 0.0f);
#else
	vec4  value = vec4(packUnorm4x8(computePixelColor().abgr), gl_FragCoord.z, 0.0f, 0.0f);
#endif

	id = int(addPixelFragCounter (coords));
	setPixelFragValue(ivec3(coords,id), value);
}