﻿#include "define.h"
// Include
vec4 computePixelColor();
// IN
#if multisample
#if multipass
	layout(binding = 10) uniform sampler2DMS tex_depth;
#endif
	layout(binding = 0) uniform sampler2DMS	tex_peel_0;
	layout(binding = 1) uniform sampler2DMS tex_peel_1;
	layout(binding = 4) uniform sampler2DMS tex_peel_2;
	layout(binding = 5) uniform sampler2DMS tex_peel_3;
	layout(binding = 6) uniform sampler2DMS tex_peel_4;
	layout(binding = 7) uniform sampler2DMS tex_peel_5;
	layout(binding = 8) uniform sampler2DMS tex_peel_6;
	layout(binding = 9) uniform sampler2DMS tex_peel_7;
#else
#if multipass
	layout(binding = 10) uniform sampler2DRect tex_depth;
#endif
	layout(binding = 0) uniform sampler2DRect tex_peel_0;
	layout(binding = 1) uniform sampler2DRect tex_peel_1;
	layout(binding = 4) uniform sampler2DRect tex_peel_2;
	layout(binding = 5) uniform sampler2DRect tex_peel_3;
	layout(binding = 6) uniform sampler2DRect tex_peel_4;
	layout(binding = 7) uniform sampler2DRect tex_peel_5;
	layout(binding = 8) uniform sampler2DRect tex_peel_6;
	layout(binding = 9) uniform sampler2DRect tex_peel_7;
#endif
// OUT
	layout(location = 0, index = 0) out vec4 out_frag_color[8];

	vec4 k[8];

#if packing
	void insertionSort(ivec2 jk, vec2 value);
#else
	void insertionSort(int   j , vec2 value);
#endif

void main(void)
{
#if multipass
	float depth;
#if multisample
	ivec2 coords = ivec2(gl_FragCoord.xy);
	depth = texelFetch(tex_depth, coords, gl_SampleID).r;
#else
	depth = texture	  (tex_depth, gl_FragCoord.xy).r;
#endif
	if(gl_FragCoord.z <= depth)
		discard;
#endif

#if multisample
	k[0] = texelFetch(tex_peel_0, coords, gl_SampleID);
	k[1] = texelFetch(tex_peel_1, coords, gl_SampleID);
	k[2] = texelFetch(tex_peel_2, coords, gl_SampleID);
	k[3] = texelFetch(tex_peel_3, coords, gl_SampleID);
	k[4] = texelFetch(tex_peel_4, coords, gl_SampleID);
	k[5] = texelFetch(tex_peel_5, coords, gl_SampleID);
	k[6] = texelFetch(tex_peel_6, coords, gl_SampleID);
	k[7] = texelFetch(tex_peel_7, coords, gl_SampleID);
#else
	k[0] = texture(tex_peel_0, gl_FragCoord.xy);
	k[1] = texture(tex_peel_1, gl_FragCoord.xy);
	k[2] = texture(tex_peel_2, gl_FragCoord.xy);
	k[3] = texture(tex_peel_3, gl_FragCoord.xy);
	k[4] = texture(tex_peel_4, gl_FragCoord.xy);
	k[5] = texture(tex_peel_5, gl_FragCoord.xy);
	k[6] = texture(tex_peel_6, gl_FragCoord.xy);
	k[7] = texture(tex_peel_7, gl_FragCoord.xy); 
#endif
    
#if packing
	vec2 value = vec2(float(packUnorm4x8(computePixelColor().abgr)), gl_FragCoord.z); 

	bool b = false;
	for(int i=0; i<KB_SIZE; i++)
	{
		for(int j=0; j<2; j++)
			if(gl_FragCoord.z < k[i][j*2 +1])
			{
				insertionSort(ivec2(i,j), value);
				b = true;
				break;
			}
		if(b)
			break;
	}

#else
	vec4 c = computePixelColor().abgr;
	vec2 value = vec2(float(packUnorm4x8(c)), gl_FragCoord.z);
	
	for(int i=0; i<KB_SIZE; i++)
	  //if(gl_FragCoord.z <  k[i].w) // old
		if(gl_FragCoord.z <= k[i].g) // treats z-fighting
		{
			insertionSort(i, value);
			break;
		}
#endif
	for(int i=0; i<KB_SIZE; i++)
		out_frag_color[i] = k[i];
}

#if packing

void insertionSort(ivec2 jk, vec2 value)
{
	int r   = KB_SIZE-1;
	int num = r-jk.x;
	for(int i=0; i<num; i++)
	{
		k[r].zw = k[r].xy;
		k[r].xy = k[r-1].zw;
		r = r-1;
	}
	
	if(jk.y==0)
	{
		k[jk.x].zw = k[jk.x].xy;
		k[jk.x].xy = value;
	}
	else
		k[jk.x].zw = value;
}

#else

void insertionSort(int j, vec2 value)
{
	int r   = KB_SIZE-1;
	int num = r-j;
	for(int i=0; i<num; i++)
	{
		k[r] = k[r-1];
		r = r-1;
	}
	k[j].rg = value;
}

#endif