﻿#include "define.h"
// IN
uniform int    layer;
uniform bool   correctAlpha;
uniform bool   useTransparency;
uniform float  transparency;

#if multisample
	layout(binding = 0) uniform sampler2DMS	tex_peel_0;
	layout(binding = 1) uniform sampler2DMS tex_peel_1;
	layout(binding = 2) uniform sampler2DMS tex_peel_2;
	layout(binding = 3) uniform sampler2DMS tex_peel_3;
	layout(binding = 4) uniform sampler2DMS tex_peel_4;
	layout(binding = 5) uniform sampler2DMS tex_peel_5;
	layout(binding = 6) uniform sampler2DMS tex_peel_6;
	layout(binding = 7) uniform sampler2DMS tex_peel_7;
#else
	layout(binding = 0) uniform sampler2DRect tex_peel_0;
	layout(binding = 1) uniform sampler2DRect tex_peel_1;
	layout(binding = 2) uniform sampler2DRect tex_peel_2;
	layout(binding = 3) uniform sampler2DRect tex_peel_3;
	layout(binding = 4) uniform sampler2DRect tex_peel_4;
	layout(binding = 5) uniform sampler2DRect tex_peel_5;
	layout(binding = 6) uniform sampler2DRect tex_peel_6;
	layout(binding = 7) uniform sampler2DRect tex_peel_7;
#endif

	layout(location = 0, index = 0) out vec4 out_frag_color;

	vec4 resolveAlphaBlend();

#if packing
	vec4 k[8];
#else
	vec2 k[8];
#endif

void main(void)
{
#if multisample
	ivec2 coords = ivec2(gl_FragCoord.xy);
#if packing
	k[0] = texelFetch(tex_peel_0, coords, gl_SampleID);
	k[1] = texelFetch(tex_peel_1, coords, gl_SampleID);
	k[2] = texelFetch(tex_peel_2, coords, gl_SampleID);
	k[3] = texelFetch(tex_peel_3, coords, gl_SampleID);
	k[4] = texelFetch(tex_peel_4, coords, gl_SampleID);
	k[5] = texelFetch(tex_peel_5, coords, gl_SampleID);
	k[6] = texelFetch(tex_peel_6, coords, gl_SampleID);
	k[7] = texelFetch(tex_peel_7, coords, gl_SampleID);
#else
	k[0] = texelFetch(tex_peel_0, coords, gl_SampleID).rg;
	k[1] = texelFetch(tex_peel_1, coords, gl_SampleID).rg;
	k[2] = texelFetch(tex_peel_2, coords, gl_SampleID).rg;
	k[3] = texelFetch(tex_peel_3, coords, gl_SampleID).rg;
	k[4] = texelFetch(tex_peel_4, coords, gl_SampleID).rg;
	k[5] = texelFetch(tex_peel_5, coords, gl_SampleID).rg;
	k[6] = texelFetch(tex_peel_6, coords, gl_SampleID).rg;
	k[7] = texelFetch(tex_peel_7, coords, gl_SampleID).rg;
#endif
#else
#if packing
	k[0] = texture(tex_peel_0, gl_FragCoord.xy);
	k[1] = texture(tex_peel_1, gl_FragCoord.xy);
	k[2] = texture(tex_peel_2, gl_FragCoord.xy);
	k[3] = texture(tex_peel_3, gl_FragCoord.xy);
	k[4] = texture(tex_peel_4, gl_FragCoord.xy);
	k[5] = texture(tex_peel_5, gl_FragCoord.xy);
	k[6] = texture(tex_peel_6, gl_FragCoord.xy);
	k[7] = texture(tex_peel_7, gl_FragCoord.xy); 
#else
	k[0] = texture(tex_peel_0, gl_FragCoord.xy).rg;
	k[1] = texture(tex_peel_1, gl_FragCoord.xy).rg;
	k[2] = texture(tex_peel_2, gl_FragCoord.xy).rg;
	k[3] = texture(tex_peel_3, gl_FragCoord.xy).rg;
	k[4] = texture(tex_peel_4, gl_FragCoord.xy).rg;
	k[5] = texture(tex_peel_5, gl_FragCoord.xy).rg;
	k[6] = texture(tex_peel_6, gl_FragCoord.xy).rg;
	k[7] = texture(tex_peel_7, gl_FragCoord.xy).rg; 
#endif
#endif

#if packing
	out_frag_color = (useTransparency) ? resolveAlphaBlend() : unpackUnorm4x8(uint(k[layer/2][(layer%2)*2])).abgr;
#else
	out_frag_color = (useTransparency) ? resolveAlphaBlend() : unpackUnorm4x8(uint(k[layer].r)).abgr;
#endif

}

#if packing
vec4 resolveAlphaBlend()
{
	float thickness=0.0f;
	vec4 color, finalColor = vec4(0.0f);
	bool b=false;

	if(correctAlpha) thickness = k[0][1]*0.5f;
	for(int i=0; i<KB_SIZE; i++)
	{
		for(int j=0; j<4; j+=2)
		{
			if(k[i][j+1] == 1.0f)
			{
				b = true;
				break;
			}
			color = vec4(unpackUnorm4x8(uint(k[i][j])).abg, transparency);
			if(correctAlpha)
			{
				if(j == KB_SIZE%2)
				{
					int i1,j1;
					j1 = j+2;
					if(j1 == 4){ i1 = i+1; j1 = j;}
					else	   { i1 = i  ;}

					thickness = (k[i1][j1+1] - k[i][j+1])*0.5f;
				}
				color.w = 1.0f - pow(1.0f - color.w, thickness*sigma);
				color.rgb *= color.w;
			}
			finalColor += color*(1.0f-finalColor.a);
		}
		if(b)
			break;
	}
	return finalColor;
}

#else

vec4 resolveAlphaBlend()
{
	float thickness=0.0f;
	vec4 color, finalColor = vec4(0.0f);

	if(correctAlpha) thickness = k[0].g*0.5f;
	for(int i=0; i<KB_SIZE; i++)
	{
		if(k[i].g == 1.0f) break;
		color = vec4(unpackUnorm4x8(uint(k[i].r)).abg,transparency);

		if(correctAlpha)
		{
			if(i%2 == KB_SIZE%2)
				thickness=(k[i+1].g-k[i].g)*0.5f;
			color.w = 1.0f - pow(1.0f - color.w, thickness*sigma);
			color.rgb *= color.w;
		}
		finalColor += color*(1.0f-finalColor.a);
	}
	return finalColor;
}
#endif