﻿#include "define.h"
#include "sort.h"
#include "resolve.h"

// IN
#if multisample
	layout(binding = 0, r32ui) readonly uniform uimage2DMS   image_next;
	layout(binding = 1, r32ui) readonly uniform uimage2DMS   image_counter;
#else
	layout(binding = 0, r32ui) readonly uniform uimage2DRect image_next;
	layout(binding = 1, r32ui) readonly uniform uimage2DRect image_counter;
#endif 
	layout(binding = 4, rg32f) readonly uniform  imageBuffer image_peel;
	layout(binding = 5, r32ui) readonly uniform uimageBuffer image_pointers;

#if multisample
	uint getPixelCurrentPageID	(const ivec2 coords) {return imageLoad(image_next	, coords, gl_SampleID).x;}
	uint getPixelFragCounter	(const ivec2 coords) {return imageLoad(image_counter, coords, gl_SampleID).x;}
#else
	uint getPixelCurrentPageID	(const ivec2 coords) {return imageLoad(image_next	, coords).x;}
	uint getPixelFragCounter	(const ivec2 coords) {return imageLoad(image_counter, coords).x;}
#endif
	uint sharedPoolGetLink		(const uint index)	 {return imageLoad (image_pointers, int(index)).x;}
	vec4 sharedPoolGetValue		(const uint index)	 {return imageLoad (image_peel	  , int(index))  ;}
	
// OUT
	layout(location = 0, index = 0) out vec4 out_frag_color;

	void main(void)
	{
		ivec2 coords = ivec2(gl_FragCoord.xy);

		uint page_id = getPixelCurrentPageID(coords);
		if(page_id > 0U)
		{
			int counter = int(getPixelFragCounter(coords));
			if(counter > 0)
			{
				for(int i=0; i<counter; i++)
				{
					fragments[i] = sharedPoolGetValue(page_id).rg;
					page_id = sharedPoolGetLink(page_id);
				}			
				out_frag_color = resolve(counter);
				return;
			}
		}
		discard;
	}