﻿#include "define.h"

// IN
uniform int    layer;
uniform float  transparency;
uniform bool   correctAlpha;
uniform bool   useTransparency;

#if multisample
	layout(binding = 0) uniform sampler2DMS	tex_peel_0;
	layout(binding = 1) uniform sampler2DMS tex_peel_1;
	layout(binding = 2) uniform sampler2DMS tex_peel_2;
	layout(binding = 3) uniform sampler2DMS tex_peel_3;
	layout(binding = 4) uniform sampler2DMS tex_peel_4;
	layout(binding = 5) uniform sampler2DMS tex_peel_5;
	layout(binding = 6) uniform sampler2DMS tex_peel_6;
	layout(binding = 7) uniform sampler2DMS tex_peel_7;
#else
	layout(binding = 0) uniform sampler2DRect tex_peel_0;
	layout(binding = 1) uniform sampler2DRect tex_peel_1;
	layout(binding = 2) uniform sampler2DRect tex_peel_2;
	layout(binding = 3) uniform sampler2DRect tex_peel_3;
	layout(binding = 4) uniform sampler2DRect tex_peel_4;
	layout(binding = 5) uniform sampler2DRect tex_peel_5;
	layout(binding = 6) uniform sampler2DRect tex_peel_6;
	layout(binding = 7) uniform sampler2DRect tex_peel_7;
#endif

// OUT
	layout(location = 0, index = 0) out  vec4 out_frag_color;

	void bubbleSort();
	vec4 resolveAlphaBlend();

#if packing
	vec2 k[KB_SIZE*2];
#else
	vec2 k[KB_SIZE];
#endif

void main(void)
{
#if multisample
#if packing
	vec4 kp;
	kp = texelFetch(tex_peel_0, ivec2(gl_FragCoord.xy), gl_SampleID);
	k[0]  = kp.xy;
	k[1]  = kp.zw;
	kp = texelFetch(tex_peel_1, ivec2(gl_FragCoord.xy), gl_SampleID);
	k[2]  = kp.xy;
	k[3]  = kp.zw;
	kp = texelFetch(tex_peel_2, ivec2(gl_FragCoord.xy), gl_SampleID);
	k[4]  = kp.xy;
	k[5]  = kp.zw;
	kp = texelFetch(tex_peel_3, ivec2(gl_FragCoord.xy), gl_SampleID);
	k[6]  = kp.xy;
	k[7]  = kp.zw;
	kp = texelFetch(tex_peel_4, ivec2(gl_FragCoord.xy), gl_SampleID);
	k[8]  = kp.xy;
	k[9]  = kp.zw;
	kp = texelFetch(tex_peel_5, ivec2(gl_FragCoord.xy), gl_SampleID);
	k[10] = kp.xy;
	k[11] = kp.zw;
	kp = texelFetch(tex_peel_6, ivec2(gl_FragCoord.xy), gl_SampleID);
	k[12] = kp.xy;
	k[13] = kp.zw;
	kp = texelFetch(tex_peel_7, ivec2(gl_FragCoord.xy), gl_SampleID);
	k[14] = kp.xy;
	k[15] = kp.zw;
#else
	k[0] = texelFetch(tex_peel_0, ivec2(gl_FragCoord.xy), gl_SampleID).rg;
	k[1] = texelFetch(tex_peel_1, ivec2(gl_FragCoord.xy), gl_SampleID).rg;
	k[2] = texelFetch(tex_peel_2, ivec2(gl_FragCoord.xy), gl_SampleID).rg;
	k[3] = texelFetch(tex_peel_3, ivec2(gl_FragCoord.xy), gl_SampleID).rg;
	k[4] = texelFetch(tex_peel_4, ivec2(gl_FragCoord.xy), gl_SampleID).rg;
	k[5] = texelFetch(tex_peel_5, ivec2(gl_FragCoord.xy), gl_SampleID).rg;
	k[6] = texelFetch(tex_peel_6, ivec2(gl_FragCoord.xy), gl_SampleID).rg;
	k[7] = texelFetch(tex_peel_7, ivec2(gl_FragCoord.xy), gl_SampleID).rg;
#endif
#else
#if packing
	vec4 kp;
	kp = texture(tex_peel_0, gl_FragCoord.xy);
	k[0]  = kp.xy;
	k[1]  = kp.zw;
	kp = texture(tex_peel_1, gl_FragCoord.xy);
	k[2]  = kp.xy;
	k[3]  = kp.zw;
	kp = texture(tex_peel_2, gl_FragCoord.xy);
	k[4]  = kp.xy;
	k[5]  = kp.zw;
	kp = texture(tex_peel_3, gl_FragCoord.xy);
	k[6]  = kp.xy;
	k[7]  = kp.zw;
	kp = texture(tex_peel_4, gl_FragCoord.xy);
	k[8]  = kp.xy;
	k[9]  = kp.zw;
	kp = texture(tex_peel_5, gl_FragCoord.xy);
	k[10] = kp.xy;
	k[11] = kp.zw;
	kp = texture(tex_peel_6, gl_FragCoord.xy);
	k[12] = kp.xy;
	k[13] = kp.zw;
	kp = texture(tex_peel_7, gl_FragCoord.xy);
	k[14] = kp.xy;
	k[15] = kp.zw;
#else
	k[0] = texture(tex_peel_0, gl_FragCoord.xy).rg;
	k[1] = texture(tex_peel_1, gl_FragCoord.xy).rg;
	k[2] = texture(tex_peel_2, gl_FragCoord.xy).rg;
	k[3] = texture(tex_peel_3, gl_FragCoord.xy).rg;
	k[4] = texture(tex_peel_4, gl_FragCoord.xy).rg;
	k[5] = texture(tex_peel_5, gl_FragCoord.xy).rg;
	k[6] = texture(tex_peel_6, gl_FragCoord.xy).rg;
	k[7] = texture(tex_peel_7, gl_FragCoord.xy).rg; 
#endif

#endif
	bubbleSort();
	out_frag_color = (useTransparency) ? resolveAlphaBlend() : unpackUnorm4x8(uint(k[layer].r)).abgr;
}

void bubbleSort()
{
	int  bb;
	vec2 temp;

#if packing
	bb = 2*KB_SIZE - 2;
#else
	bb =   KB_SIZE - 2;
#endif
	for (int i = bb; i >= 0; --i) 
		for (int j = 0; j <= i; ++j)
			if (k[j].g > k[j+1].g)
			{
				temp   = k[j];
				k[j]   = k[j+1];
				k[j+1] = temp;
			}
}

vec4 resolveAlphaBlend()
{
	int   bb;
	float thickness=0.0f;
	vec4  color, finalColor = vec4(0.0f);

#if packing
	bb = 2*KB_SIZE;
#else
	bb =   KB_SIZE;
#endif
	if(correctAlpha) thickness = k[0].g*0.5f;
	for(int i=0; i<bb; i++)
	{
		if(k[i].g == 1.0f)
			break;
		color = vec4(unpackUnorm4x8(uint(k[i].r)).abg, transparency);
		if(correctAlpha)
		{
			if(i%2 == bb%2)
				thickness=(k[i+1].g - k[i].g)*0.5f;
			color.a    = 1.0f - pow(1.0f - color.a, thickness*sigma);
			color.rgb *= color.a;
		}
		finalColor += color*(1.0f-finalColor.a);
	}
	return finalColor;
}