﻿#include "define.h"
#include "s-buffer.h"

	coherent uniform uint *next_address[COUNTERS];

#if multisample
#if atomic
	layout(binding = 0, r32ui) readonly  uniform uimage2DMS    image_counter;
#else
	layout(binding = 0)					 uniform sampler2DMS   tex_counter;
#endif
	layout(binding = 1, r32ui) writeonly uniform uimage2DMS    image_next;
#else
#if atomic
	layout(binding = 0, r32ui) readonly  uniform uimage2DRect  image_counter;
#else
	layout(binding = 0)					 uniform sampler2DRect tex_counter;
#endif
	layout(binding = 1, r32ui) writeonly uniform uimage2DRect  image_next;
#endif 

	uint addSharedNextAddress (int j	   , uint val) { return atomicAdd	(next_address[j], val);}
#if multisample
	void setPixelNextAddress  (ivec2 coords, uint val) {		imageStore	(image_next	  , coords, gl_SampleID, uvec4(val, 0U, 0U, 0U) );}
#if atomic
	uint getPixelFragCounter  (ivec2 coords			 ) { return imageLoad	(image_counter, coords, gl_SampleID).x ;}
#else
	uint getPixelFragCounter  (ivec2 coords			 ) { return uint(texture(tex_counter  , coords, gl_SampleID).x);}
#endif
#else
	void setPixelNextAddress  (ivec2 coords, uint val) {		imageStore	(image_next	  , coords, uvec4(val, 0U, 0U, 0U) );}
#if atomic	
	uint getPixelFragCounter  (ivec2 coords			 ) { return imageLoad   (image_counter, coords).x ;}
#else
	uint getPixelFragCounter  (ivec2 coords			 ) { return uint(texture(tex_counter  , coords).x);}
#endif
#endif	
	
void main(void)
{
	ivec2 coords = ivec2(gl_FragCoord.xy);

	uint counter = getPixelFragCounter(coords);
	if(counter == 0U)
		discard;

	int  hash_id = coords.y % COUNTERS;							// old hash function: [hash_id = (coords.x + WIDTH*coords.y) % COUNTERS]
	uint address = addSharedNextAddress (hash_id, counter);
	setPixelNextAddress (coords, address);
}