﻿#include "define.h"
#include "s-buffer.h"
#include "sort.h"
#include "resolve.h"

// IN
	uniform uint  total_fragments;
	uniform uint *final_address[COUNTERS];
//
#if multisample
#if atomic
	layout(binding = 0, r32ui) readonly  uniform uimage2DMS    image_counter;
#else
	layout(binding = 0)					 uniform sampler2DMS   tex_counter;
#endif
	layout(binding = 1, r32ui) readonly  uniform uimage2DMS    image_next;
#else
#if atomic
	layout(binding = 0, r32ui) readonly  uniform uimage2DRect  image_counter;
#else
	layout(binding = 0)					 uniform sampler2DRect tex_counter;
#endif
	layout(binding = 1, r32ui) readonly  uniform uimage2DRect  image_next;
#endif 
	layout(binding = 4, rg32f) readonly  uniform imageBuffer   image_peel;

	vec4 sharedPoolGetValue	  (const uint index				   ) {return imageLoad (image_peel, int(index));}

#if multisample
	uint getPixelNextAddress  (const ivec2 coords){return imageLoad(image_next   , coords, gl_SampleID).x-1U;}
#if atomic
	uint getPixelFragCounter  (const ivec2 coords){return imageLoad(image_counter, coords, gl_SampleID).x;   }
#else
	uint getPixelFragCounter  (const ivec2 coords){return uint(texture(tex_counter, coords, gl_SampleID).x); }
#endif
#else
	uint getPixelNextAddress  (const ivec2 coords){return imageLoad(image_next   , coords).x-1U;}
#if atomic
	uint getPixelFragCounter  (const ivec2 coords){return imageLoad(image_counter, coords).x;   }
#else
	uint getPixelFragCounter  (const ivec2 coords){return uint(texture(tex_counter, coords).x); }
#endif
#endif
	// OUT
	layout(location = 0, index = 0) out vec4 out_frag_color;

	void main(void)
	{
		ivec2 coords = ivec2(gl_FragCoord.xy);

		int counter = int(getPixelFragCounter(coords));
		if(counter > 0)
		{	
			uint address = getPixelNextAddress(coords);
			int  hash_id = coords.y % COUNTERS;
	
#if inverse	
			bool front_prefix = hash_id < COUNTERS2 ? true : false;
#else
			bool front_prefix = true;
#endif
			uint sum = (*final_address[hash_id]);

			uint page_id   = front_prefix ? address + sum : total_fragments - address - sum;
			uint direction = front_prefix ? -1U : 1U;

			for(int i=0; i<counter; i++)
			{
				fragments[i] = sharedPoolGetValue(page_id).rg;
				page_id		+= direction;
			}			
			out_frag_color = resolve(counter);
		}
		else
			discard;
	}