﻿#include "define.h"

// IN
uniform int    layer;
uniform float  transparency;
uniform bool   correctAlpha;
uniform bool   useTransparency;

layout(binding = 0) uniform sampler2DMS tex_peel;

// OUT
layout(location = 0, index = 0) out  vec4 out_frag_color;

void bubbleSort		  ();
vec4 resolveAlphaBlend();
vec2 k[STENCIL_SAMPLES];

void main(void)
{
	for(int i=0; i<STENCIL_SAMPLES; i++)
		k[i] = texelFetch(tex_peel, ivec2(gl_FragCoord.xy), i).rg;
	bubbleSort();

	out_frag_color = (useTransparency) ? resolveAlphaBlend() : unpackUnorm4x8(uint(k[layer].r)).abgr;
}

void bubbleSort()
{
	vec2 tmp;
	for (int i = (STENCIL_SAMPLES - 2); i >= 0; --i) 
		for (int j = 0; j <= i; ++j)
			if (k[j].g > k[j+1].g)
			{
				tmp	   = k[j];
				k[j]   = k[j+1];
				k[j+1] = tmp;
			}
}

vec4 resolveAlphaBlend()
{
	float thickness=0.0f;
	vec4 color, finalColor = vec4(0.0f);

	if(correctAlpha) thickness = k[0].g*0.5f;
	for(int i=0; i<STENCIL_SAMPLES; i++)
	{
		if(k[i].g == 1.0f)
			break;

		color = vec4(unpackUnorm4x8(uint(k[i].r)).abg,transparency);
		if(correctAlpha)
		{
			if(i%2 == STENCIL_SAMPLES%2)
				thickness=(k[i+1].g-k[i].g)*0.5f;
			color.w	   = 1.0f - pow(1.0f - color.w, thickness*sigma);
			color.rgb *= color.a;
		}
		finalColor += color*(1.0f-finalColor.a);
	}
	return finalColor;
}